@@ -1,17 +1,24 @@
-# ================================================================
+# ============================================================================
 package Business::UPS::Tracking;
-# ================================================================
+# ============================================================================
 use utf8;
-use Moose;
 use 5.0100;
 
+use metaclass (
+    metaclass   => "Moose::Meta::Class",
+    error_class => "Business::UPS::Tracking::Exception",
+);
+use Moose;
+
 use LWP::UserAgent;
 use Business::UPS::Tracking::Utils;
 use Business::UPS::Tracking::Request;
 
 use version;
-use vars qw($VERSION);
-$VERSION = version->new('1.01');
+
+our $VERSION = version->new('1.03');
+our $AUTHORITY = 'cpan:MAROS';
+our $CHECKSUM = 1;
 
 =encoding utf8
 
@@ -24,16 +31,22 @@ Business::UPS::Tracking - Interface to the UPS tracking webservice
   use Business::UPS::Tracking;
   
   my $tracking = Business::UPS::Tracking->new(
-    license  => '1CFFED5A5E91B17',
-    username => 'myupsuser',
-    password => 'secret',
+    AccessLicenseNumber => '1CFFED5A5E91B17',
+    UserId              => 'myupsuser',
+    Password            => 'secret',
   );
   
   eval {
     my $response = $tracking->request(
       TrackingNumber  => '1Z12345E1392654435',
     )->run();
-    say 'Delivery scheduled for '.$response->shipment->ScheduledDelivery->dmy();
+    
+    foreach my $shipment ($response->shipment) {
+        say 'Service code is '.$shipment->ServiceCode;
+        foreach my $package ($shipment->Package) {
+            say 'Status is '.$package->CurrentStatus;
+        }
+    }
   };
   
   if (my $e = Exception::Class->caught) {
@@ -42,7 +55,7 @@ Business::UPS::Tracking - Interface to the UPS tracking webservice
         say 'HTTP ERROR:'.$e->full_message;
       }
       when ($_->isa('Business::UPS::Tracking::X::UPS')) {
-        say 'DPD ERRPR:'.$e->full_message.' ('.$e->code.')';
+        say 'UPS ERROR:'.$e->full_message.' ('.$e->code.')';
       }
       default {
         say 'SOME ERROR:'.$e;
@@ -105,13 +118,11 @@ documentation for documentation how to catch and rethrow exeptions.
 
 The following exception classes are defined:
 
-=over
-
-=item * Business::UPS::Tracking::X
+=head3 Business::UPS::Tracking::X
 
 Basic exception class. All other exception classes inherit from this class.
 
-=item * Business::UPS::Tracking::X::HTTP
+=head3 Business::UPS::Tracking::X::HTTP
 
 HTTP error. The object provides additional parameters:
 
@@ -123,7 +134,7 @@ HTTP error. The object provides additional parameters:
 
 =back
 
-=item * Business::UPS::Tracking::X::UPS
+=head3 Business::UPS::Tracking::X::UPS
 
 UPS error message.The object provides additional parameters:
 
@@ -139,15 +150,20 @@ UPS error message.The object provides additional parameters:
 
 =back
 
-=item * Business::UPS::Tracking::X::XML
+=head3 Business::UPS::Tracking::X::XML
 
 XML parser or schema error.
 
+=head3 Business::UPS::Tracking::X::CLASS
+
+Error originating from the wrong usage of a method/accessor/class. Most
+commonly this will be thrown because of a failing type constraint.
+
 =back
 
-=head2 Accessor / method nameing
+=head2 Accessor / method naming
 
-The nameing of the methods and accessors tries to stick close to the names
+The naming of the methods and accessors tries to stick close to the names
 used by the UPS webservice. All accessors containg uppercase letters access
 xml data. Lowercase-only accessors and methods are used for utility 
 functions.
@@ -156,7 +172,7 @@ functions.
 
 In order to use this module you need to obtain a "Tracking WebService" 
 license key. See L<http://www.ups.com/e_comm_access/gettools_index> for more
-inforation.
+information.
 
 =head1 METHODS
 
@@ -222,25 +238,30 @@ has 'AccessLicenseNumber' => (
     is       => 'rw',
     required => 1,
     isa      => 'Str',
+    documentation   => 'UPS webservice license number',
 );
 has 'UserId' => (
     is       => 'rw',
     required => 1,
     isa      => 'Str',
+    documentation   => 'UPS webservice user id',
 );
 has 'Password' => (
     is       => 'rw',
     required => 1,
     isa      => 'Str',
+    documentation   => 'UPS webservice password',
 );
 has 'retry_http' => (
     is      => 'rw',
     isa     => 'Int',
     default => 0,
+    documentation   => 'Number of retries if HTTP erros occur [Default 0]',
 );
 has 'url' => (
     is      => 'rw',
-    default => 'https://wwwcie.ups.com/ups.app/xml/Track'
+    default => 'https://wwwcie.ups.com/ups.app/xml/Track',
+    documentation   => 'UPS webservice url',
 );
 has '_ua' => (
     is      => 'rw',
@@ -326,7 +347,7 @@ this module.
 
 =head1 SEE ALSO
 
-Download the UPS ""OnLine® Tools Tracking Developer Guide"" and get a
+Download the UPS "OnLine® Tools Tracking Developer Guide" and get a
 developer key at L<http://www.ups.com/e_comm_access/gettools_index?loc=en_US>. 
 Please check the "Developer Guide" for more detailed documentation on the
 various fields.
